@@ -3,7 +3,7 @@
 // SPDX-FileContributor: Gerhard de Clercq <gerhard.declercq@kdab.com>
 //
 // SPDX-License-Identifier: MIT OR Apache-2.0
-use crate::utils::{is_type_ident_ptr, type_to_namespace};
+use crate::utils::type_to_namespace;
 use convert_case::{Case, Casing};
 use derivative::*;
 use proc_macro2::{Span, TokenStream};
@@ -11,7 +11,7 @@ use std::result::Result;
 use syn::{spanned::Spanned, *};
 
 /// Describes an ident which has a different name in C++ and Rust
-#[derive(Debug)]
+#[derive(Debug, PartialEq)]
 pub(crate) struct CppRustIdent {
     /// The ident for C++
     pub(crate) cpp_ident: Ident,
@@ -21,33 +21,53 @@ pub(crate) struct CppRustIdent {
 
 /// Describes a Qt type
 #[derive(Debug, PartialEq)]
-pub enum QtTypes {
+pub(crate) enum QtTypes {
     Bool,
+    /// A CppObj which is being passed as a parameter in a method
+    CppObj {
+        /// Whether this CppObj is from the current module or another module
+        external: bool,
+        /// The ident of the type for C++, eg the MyObject or CppObj from:
+        /// C++ - MyObject or cxx_qt::sub_object::CppObj
+        cpp_type_idents: Vec<Ident>,
+        /// A cache of the C++ type as a string with the namespace
+        /// eg "cxx_qt::module::CppObj"
+        cpp_type_idents_string: String,
+        /// The ident of the type for Rust, eg the MyObject or CppObj from:
+        /// Rust - MyObject or crate::sub_object::CppObj
+        rust_type_idents: Vec<Ident>,
+        /// The combined name of the type, this is a single word which is used
+        /// as the type name of the C++ type in the Rust cxx bridge
+        /// eg MyObject or SubObject
+        /// This needs the match the name of the C++ class without namespace
+        /// as we add this in the macro attribute
+        //
+        // TODO: later can we use the fully qualified path
+        // eg crate::my_module::CppObj to Crate_MyModule_CppObj?
+        combined_name: Ident,
+    },
+    DateTime,
+    QDateTime,
     F32,
     F64,
     I8,
     I16,
     I32,
-    Pin {
-        /// Cache of the last type_idents as a str with it's namespace for C++ to reference
-        ident_namespace_str: String,
-        /// Whether the inner type of this Pin is mut
-        is_mut: bool,
-        /// Whether the inner type of this Pin is the current type, and therefore "this" in C++
-        is_this: bool,
-        /// The ident of the inner type (eg the T of Pin<T>)
-        type_idents: Vec<Ident>,
-    },
-    Ptr {
-        /// Cache of the last type ident as a str C++ and Rust to reference
-        ident_str: String,
-        /// Cache of the last type ident as a str with it's namespace for C++ to reference
-        ident_namespace_str: String,
-    },
+    QColor,
+    Color,
+    QDate,
+    QPoint,
     QPointF,
+    QRect,
+    QRectF,
+    QSize,
+    QSizeF,
     QString,
     String,
     Str,
+    QTime,
+    QUrl,
+    Url,
     QVariant,
     Variant,
     U8,
@@ -56,15 +76,29 @@ pub enum QtTypes {
     Unknown,
 }
 
+impl QtTypes {
+    /// Whether this type is opaque so will be a UniquePtr<T> when returned from Rust to C++
+    pub(crate) fn is_opaque(&self) -> bool {
+        match self {
+            Self::QColor | Self::Color => true,
+            Self::QDateTime | Self::DateTime => true,
+            Self::QString | Self::String | Self::Str => true,
+            Self::QUrl | Self::Url => true,
+            Self::QVariant | Self::Variant => true,
+            _others => false,
+        }
+    }
+}
+
 /// Describes a type
 #[derive(Debug)]
 pub(crate) struct ParameterType {
     /// The type of the parameter
     pub(crate) idents: Vec<Ident>,
+    /// If this parameter is mutable
+    pub(crate) is_mut: bool,
     /// If this parameter is a reference
     pub(crate) is_ref: bool,
-    /// The original type, this allows us to annotate an error with a span later
-    pub(crate) original_ty: syn::Type,
     /// The detected Qt type of the parameter
     pub(crate) qt_type: QtTypes,
 }
@@ -84,6 +118,8 @@ pub(crate) struct Parameter {
 pub(crate) struct Invokable {
     /// The ident of the function
     pub(crate) ident: CppRustIdent,
+    /// If the invokable needs a wrapper, this is it's ident
+    pub(crate) ident_wrapper: Option<CppRustIdent>,
     /// The parameters that the function takes in
     pub(crate) parameters: Vec<Parameter>,
     /// The return type information
@@ -148,16 +184,6 @@ enum ExtractTypeIdentError {
     IdentEmpty(Span),
     /// There are multiple idents but didn't start with crate::
     UnknownAndNotCrate(Span),
-    /// There is a Pin<T> but the T is unknown to our converters
-    UnknownPinType(Span),
-}
-
-/// Describe what the extract method should do when parsing an unknown type
-pub enum ExtractMethodUnknownOptions {
-    /// Do not error, add unknown as the QtType, and continue
-    IgnoreUnknownTypes,
-    /// Error when there is an unknown, as all types must be parsed
-    ParseAllTypes,
 }
 
 /// Extract the Qt type from a list of Ident's
@@ -165,6 +191,7 @@ fn extract_qt_type(
     idents: &[Ident],
     original_ty: &syn::Type,
     cpp_namespace_prefix: &[&str],
+    qt_ident: &Ident,
 ) -> Result<QtTypes, ExtractTypeIdentError> {
     // TODO: can we support generic Qt types as well eg like QObject or QAbstractListModel?
     // so that QML can set a C++/QML type into the property ? or is that not useful?
@@ -177,15 +204,35 @@ fn extract_qt_type(
         // We can assume that idents has an entry at index zero, because there is one entry
         match idents[0].to_string().as_str() {
             "bool" => Ok(QtTypes::Bool),
+            "CppObj" => Ok(QtTypes::CppObj {
+                external: false,
+                cpp_type_idents: vec![qt_ident.clone()],
+                cpp_type_idents_string: qt_ident.to_string(),
+                rust_type_idents: vec![qt_ident.clone()],
+                combined_name: qt_ident.clone(),
+            }),
+            "DateTime" => Ok(QtTypes::DateTime),
+            "QDateTime" => Ok(QtTypes::QDateTime),
             "f32" => Ok(QtTypes::F32),
             "f64" => Ok(QtTypes::F64),
             "i8" => Ok(QtTypes::I8),
             "i16" => Ok(QtTypes::I16),
             "i32" => Ok(QtTypes::I32),
+            "QColor" => Ok(QtTypes::QColor),
+            "Color" => Ok(QtTypes::Color),
+            "QDate" => Ok(QtTypes::QDate),
+            "QPoint" => Ok(QtTypes::QPoint),
             "QPointF" => Ok(QtTypes::QPointF),
+            "QRect" => Ok(QtTypes::QRect),
+            "QRectF" => Ok(QtTypes::QRectF),
+            "QSize" => Ok(QtTypes::QSize),
+            "QSizeF" => Ok(QtTypes::QSizeF),
             "QString" => Ok(QtTypes::QString),
             "str" => Ok(QtTypes::Str),
             "String" => Ok(QtTypes::String),
+            "QTime" => Ok(QtTypes::QTime),
+            "QUrl" => Ok(QtTypes::QUrl),
+            "Url" => Ok(QtTypes::Url),
             "QVariant" => Ok(QtTypes::QVariant),
             "Variant" => Ok(QtTypes::Variant),
             "u8" => Ok(QtTypes::U8),
@@ -193,19 +240,36 @@ fn extract_qt_type(
             "u32" => Ok(QtTypes::U32),
             _other => Ok(QtTypes::Unknown),
         }
-    // As this type ident has more than one segment, check if it is a pointer
-    } else if is_type_ident_ptr(idents) {
-        // We can assume that last exists as there is at least one entry in idents, so unwrap() is fine here
-        let ident_str = idents.last().unwrap().to_string();
-        let mut namespace = type_to_namespace(cpp_namespace_prefix, idents)
-            // TODO: should we have our own error type? UnknownPtrNamespace?
-            .map_err(|_| ExtractTypeIdentError::InvalidType(original_ty.span()))?;
-        namespace.push(ident_str.clone());
-
-        Ok(QtTypes::Ptr {
-            ident_str,
-            // TODO: do we need to track is_ref here?
-            ident_namespace_str: namespace.join("::"),
+    // If the first ident is crate, the last is CppObj, and we have more than two parts
+    // then we are an external CppObj
+    } else if idents.len() > 2
+        && idents.first().unwrap().to_string().as_str() == "crate"
+        && idents.last().unwrap().to_string().as_str() == "CppObj"
+    {
+        let cpp_type_idents = cpp_namespace_prefix
+            .to_vec()
+            .iter()
+            .map(|s| quote::format_ident!("{}", s))
+            // TODO: once we generate sub folders for nested modules, this will need to use all
+            // type idents other than first and last. as the namespace will then reflect sub dirs
+            // https://github.com/KDAB/cxx-qt/issues/19
+            .chain(idents.iter().skip(idents.len() - 2).cloned())
+            .collect::<Vec<Ident>>();
+        Ok(QtTypes::CppObj {
+            external: true,
+            cpp_type_idents_string: cpp_type_idents
+                .iter()
+                .map(|ident| ident.to_string())
+                .collect::<Vec<String>>()
+                .join("::"),
+            cpp_type_idents,
+            rust_type_idents: idents.to_vec(),
+            // TODO: later can we use the fully qualified path
+            // eg crate::my_module::CppObj to Crate_MyModule_CppObj?
+            combined_name: quote::format_ident!(
+                "{}",
+                idents[idents.len() - 2].to_string().to_case(Case::Pascal)
+            ),
         })
     // This is an unknown type that did not start with crate and has multiple parts
     } else {
@@ -236,22 +300,28 @@ fn path_to_idents(path: &syn::Path) -> Result<Vec<Ident>, ExtractTypeIdentError>
 fn extract_type_ident(
     ty: &syn::Type,
     cpp_namespace_prefix: &[&str],
+    qt_ident: &Ident,
 ) -> Result<ParameterType, ExtractTypeIdentError> {
     // Temporary storage of the current syn::TypePath if one is found
     let ty_path;
+    let is_mut;
     // Whether this syn::Type is a reference or not
     let is_ref;
 
     match ty {
         // The type is simply a path (eg std::slice::Iter)
         Type::Path(path) => {
+            is_mut = false;
             is_ref = false;
             ty_path = path;
         }
         // The type is a reference, so see if it contains a path
-        Type::Reference(TypeReference { elem, .. }) => {
+        Type::Reference(TypeReference {
+            mutability, elem, ..
+        }) => {
             // If the type is a path then extract it and mark is_ref
             if let Type::Path(path) = &**elem {
+                is_mut = mutability.is_some();
                 is_ref = true;
                 ty_path = path;
             } else {
@@ -263,112 +333,17 @@ fn extract_type_ident(
         }
     }
 
-    // Check if this type is a Pin<T>, if it is then attempt to extract it
-    if let Some(segment) = ty_path.path.segments.first() {
-        if segment.ident.to_string().as_str() == "Pin" {
-            match &segment.arguments {
-                PathArguments::AngleBracketed(AngleBracketedGenericArguments { args, .. })
-                    if args.len() == 1 =>
-                {
-                    let is_mut;
-                    let ty_path;
-
-                    // We have already checked that args is of len 1
-                    match &args[0] {
-                        // We are &mut T
-                        GenericArgument::Type(Type::Reference(TypeReference {
-                            elem,
-                            mutability,
-                            ..
-                        })) => {
-                            is_mut = mutability.is_some();
-
-                            if let Type::Path(path) = &**elem {
-                                ty_path = path;
-                            } else {
-                                return Err(ExtractTypeIdentError::UnknownPinType(ty.span()));
-                            }
-                        }
-                        // TODO: later we might want extra cases to handle non ref versions? Pin<T>
-                        _others => {
-                            return Err(ExtractTypeIdentError::UnknownPinType(ty.span()));
-                        }
-                    }
-
-                    // Convert our inner type to a list of idents
-                    let type_idents = path_to_idents(&ty_path.path)?;
-                    // Create the Qt type for the Pin
-                    let qt_type = QtTypes::Pin {
-                        // For a given Pin<crate::A::T> we want to make it cxx_qt::A::T
-                        ident_namespace_str: if let Some(ident) = type_idents.last() {
-                            let ident_str = ident.to_string();
-                            // Ignore Pin<&mut CppObj> from our namespaces for now
-                            //
-                            // TODO: does this need to be considered?
-                            if type_idents.len() == 1 && ident_str.as_str() == "CppObj" {
-                                ident_str
-                            } else {
-                                // Extract the namespace of the type
-                                let mut namespace =
-                                    type_to_namespace(cpp_namespace_prefix, &type_idents)
-                                        // TODO: should we have our own error type? UnknownPtrNamespace?
-                                        .map_err(|_| {
-                                            ExtractTypeIdentError::InvalidType(ty_path.span())
-                                        })?;
-                                namespace.push(ident_str);
-                                namespace.join("::")
-                            }
-                        } else {
-                            // There was no T part of Pin<T>
-                            //
-                            // TODO: could be it's own enum error? InvalidPinType?
-                            return Err(ExtractTypeIdentError::UnknownPinType(ty.span()));
-                        },
-                        is_mut,
-                        // If the T in Pin<T> is CppObj, then it is "this"
-                        is_this: if let Some(ident) = type_idents.first() {
-                            ident.to_string().as_str() == "CppObj"
-                        } else {
-                            false
-                        },
-                        type_idents,
-                    };
-                    // We put the Pin as our idents, the inner type goes into the QtType
-                    //
-                    // The gen_cpp and gen_rs don't use this as they'll special case Pin<T> when
-                    // it's an invokable argument to use the inner type.
-                    // It is only used in return types or Q_PROPERTY in gen_rs
-                    //
-                    // TODO: should Pin<T> be accepted for return types and Q_PROPERTY?
-                    let idents = vec![segment.ident.to_owned()];
-
-                    return Ok(ParameterType {
-                        // Read each of the path segment to turn a &syn::TypePath of std::slice::Iter
-                        // into an owned Vec<Ident>
-                        idents,
-                        is_ref,
-                        // We need to have the original type so that errors can Span if there are no idents
-                        original_ty: ty.to_owned(),
-                        qt_type,
-                    });
-                }
-                _ => {}
-            }
-        }
-    }
-
     let idents = path_to_idents(&ty_path.path)?;
     // Extract the Qt type this is used in C++ and Rust generation
-    let qt_type = extract_qt_type(&idents, ty, cpp_namespace_prefix)?;
+    let qt_type = extract_qt_type(&idents, ty, cpp_namespace_prefix, qt_ident)?;
 
     // Create and return a ParameterType
     Ok(ParameterType {
         // Read each of the path segment to turn a &syn::TypePath of std::slice::Iter
         // into an owned Vec<Ident>
         idents,
+        is_mut,
         is_ref,
-        // We need to have the original type so that errors can Span if there are no idents
-        original_ty: ty.to_owned(),
         qt_type,
     })
 }
@@ -386,6 +361,7 @@ struct ExtractedInvokables {
 fn extract_invokables(
     items: &[ImplItem],
     cpp_namespace_prefix: &[&str],
+    qt_ident: &Ident,
 ) -> Result<ExtractedInvokables, TokenStream> {
     let mut invokables = Vec::new();
     let mut normal_methods = Vec::new();
@@ -400,12 +376,11 @@ fn extract_invokables(
         //
         // TODO: later should we pass through unknown items
         // or should they have an attribute to ignore
-        let mut method;
-        if let ImplItem::Method(m) = item {
-            method = m.clone();
+        let mut method = if let ImplItem::Method(m) = item {
+            m.clone()
         } else {
             return Err(Error::new(item.span(), "Only methods are supported.").to_compile_error());
-        }
+        };
 
         let filtered_attrs: Vec<syn::Attribute> = method
             .attrs
@@ -432,7 +407,7 @@ fn extract_invokables(
         method.attrs = filtered_attrs;
 
         // Extract the ident, parameters, return type of the method
-        let invokable = extract_invokable(&method, cpp_namespace_prefix)?;
+        let invokable = extract_invokable(&method, cpp_namespace_prefix, qt_ident)?;
         invokables.push(invokable);
     }
 
@@ -446,7 +421,7 @@ fn extract_invokables(
 pub(crate) fn extract_method_params(
     method: &ImplItemMethod,
     cpp_namespace_prefix: &[&str],
-    options: ExtractMethodUnknownOptions,
+    qt_ident: &Ident,
 ) -> Result<Vec<Parameter>, TokenStream> {
     method.sig.inputs
         .iter()
@@ -459,66 +434,39 @@ pub(crate) fn extract_method_params(
             // TODO: does this mean that if self is Typed we need to skip it?
             // so should we ignore the first parameter if it is named "self"?
             if let FnArg::Typed(PatType { pat, ty, .. }) = parameter {
-                // The name ident of the parameter
-                let parameter_ident;
                 // The type ident of the parameter
                 let type_ident;
 
                 // Try to extract the name of the parameter
-                if let Pat::Ident(PatIdent { ident, .. }) = &**pat {
-                    parameter_ident = ident;
+                let parameter_ident = if let Pat::Ident(PatIdent { ident, .. }) = &**pat {
+                    ident
                 } else {
                     return Err(
                         Error::new(parameter.span(), "Invalid argument ident format.")
                             .to_compile_error(),
                     );
-                }
+                };
 
                 // Try to extract the type of the parameter
-                let extracted_type = extract_type_ident(ty, cpp_namespace_prefix);
-
-                match options {
-                    ExtractMethodUnknownOptions::IgnoreUnknownTypes => {
-                        match extracted_type {
-                            Ok(result) => type_ident = result,
-                            Err(_) => type_ident = ParameterType {
-                                // FIXME: does it matter that we don't have real values here
-                                // for idents and is_ref
-                                //
-                                // As this is only currently used when passing through
-                                // rust obj methods which aren't invokables
-                                idents: vec![],
-                                is_ref: false,
-                                original_ty: (**ty).to_owned(),
-                                qt_type: QtTypes::Unknown
-                            }
-                        }
-                    },
-                    ExtractMethodUnknownOptions::ParseAllTypes => {
-                        match extracted_type {
-                            Ok(result) => type_ident = result,
-                            Err(ExtractTypeIdentError::InvalidArguments(span)) => {
-                                return Err(Error::new(
-                                    span,
-                                    "Argument should not be angle bracketed or parenthesized.",
-                                )
-                                .to_compile_error());
-                            }
-                            Err(ExtractTypeIdentError::InvalidType(span)) => {
-                                return Err(
-                                    Error::new(span, "Invalid argument ident format.").to_compile_error()
-                                )
-                            }
-                            Err(ExtractTypeIdentError::IdentEmpty(span)) => {
-                                return Err(Error::new(span, "Argument type ident must have at least one segment").to_compile_error())
-                            }
-                            Err(ExtractTypeIdentError::UnknownAndNotCrate(span)) => {
-                                return Err(Error::new(span, "First argument type ident segment must start with 'crate' if there are multiple").to_compile_error())
-                            }
-                            Err(ExtractTypeIdentError::UnknownPinType(span)) => {
-                                return Err(Error::new(span, "Unknown argument Pin<T> type ident to parse").to_compile_error())
-                            }
-                        }
+                match extract_type_ident(ty, cpp_namespace_prefix, qt_ident) {
+                    Ok(result) => type_ident = result,
+                    Err(ExtractTypeIdentError::InvalidArguments(span)) => {
+                        return Err(Error::new(
+                            span,
+                            "Argument should not be angle bracketed or parenthesized.",
+                        )
+                        .to_compile_error());
+                    }
+                    Err(ExtractTypeIdentError::InvalidType(span)) => {
+                        return Err(
+                            Error::new(span, "Invalid argument ident format.").to_compile_error()
+                        )
+                    }
+                    Err(ExtractTypeIdentError::IdentEmpty(span)) => {
+                        return Err(Error::new(span, "Argument type ident must have at least one segment").to_compile_error())
+                    }
+                    Err(ExtractTypeIdentError::UnknownAndNotCrate(span)) => {
+                        return Err(Error::new(span, "First argument type ident segment must start with 'crate' if there are multiple").to_compile_error())
                     }
                 }
 
@@ -544,19 +492,16 @@ pub(crate) fn extract_method_params(
 fn extract_invokable(
     method: &ImplItemMethod,
     cpp_namespace_prefix: &[&str],
+    qt_ident: &Ident,
 ) -> Result<Invokable, TokenStream> {
     let method_ident = &method.sig.ident;
     let output = &method.sig.output;
 
-    let parameters = extract_method_params(
-        method,
-        cpp_namespace_prefix,
-        ExtractMethodUnknownOptions::ParseAllTypes,
-    )?;
+    let parameters = extract_method_params(method, cpp_namespace_prefix, qt_ident)?;
 
     let return_type = if let ReturnType::Type(_, ty) = output {
         // This output has a return type, so extract the type
-        match extract_type_ident(ty, cpp_namespace_prefix) {
+        match extract_type_ident(ty, cpp_namespace_prefix, qt_ident) {
             Ok(result) => Some(result),
             Err(ExtractTypeIdentError::InvalidArguments(span)) => {
                 return Err(Error::new(
@@ -581,12 +526,6 @@ fn extract_invokable(
                 )
                 .to_compile_error())
             }
-            Err(ExtractTypeIdentError::UnknownPinType(span)) => {
-                return Err(
-                    Error::new(span, "Unknown return Pin<T> type ident to parse")
-                        .to_compile_error(),
-                )
-            }
         }
     } else {
         None
@@ -597,9 +536,25 @@ fn extract_invokable(
         cpp_ident: quote::format_ident!("{}", ident_str.to_case(Case::Camel)),
         rust_ident: quote::format_ident!("{}", ident_str.to_case(Case::Snake)),
     };
+    // We need a wrapper for any opaque types or pointers in the parameters or return types
+    let ident_wrapper = if return_type
+        .as_ref()
+        .map_or(false, |return_type| return_type.qt_type.is_opaque())
+        || parameters
+            .iter()
+            .any(|parameter| matches!(parameter.type_ident.qt_type, QtTypes::CppObj { .. }))
+    {
+        Some(CppRustIdent {
+            cpp_ident: quote::format_ident!("{}Wrapper", ident_method.cpp_ident),
+            rust_ident: quote::format_ident!("{}_wrapper", ident_method.rust_ident),
+        })
+    } else {
+        None
+    };
 
     Ok(Invokable {
         ident: ident_method,
+        ident_wrapper,
         parameters,
         return_type,
         original_method: method.to_owned(), // TODO: remove to_owned once extract_invokable is split
@@ -610,6 +565,7 @@ fn extract_invokable(
 fn extract_properties(
     s: &ItemStruct,
     cpp_namespace_prefix: &[&str],
+    qt_ident: &Ident,
 ) -> Result<Vec<Property>, TokenStream> {
     let mut properties = Vec::new();
 
@@ -640,7 +596,7 @@ fn extract_properties(
                 // Extract the type of the field
                 let type_ident;
 
-                match extract_type_ident(ty, cpp_namespace_prefix) {
+                match extract_type_ident(ty, cpp_namespace_prefix, qt_ident) {
                     Ok(result) => type_ident = result,
                     Err(ExtractTypeIdentError::InvalidArguments(span)) => {
                         return Err(Error::new(
@@ -660,9 +616,6 @@ fn extract_properties(
                     Err(ExtractTypeIdentError::UnknownAndNotCrate(span)) => {
                         return Err(Error::new(span, "First named field type ident segment must start with 'crate' if there are multiple").to_compile_error())
                     }
-                    Err(ExtractTypeIdentError::UnknownPinType(span)) => {
-                        return Err(Error::new(span, "Unknown named field Pin<T> type ident to parse").to_compile_error())
-                    }
                 }
 
                 // Build the getter/setter/notify idents with their Rust and C++ idents
@@ -855,8 +808,11 @@ pub fn extract_qobject(
                                     _others => original_trait_impls.push(original_impl.to_owned()),
                                 }
                             } else {
-                                let mut extracted =
-                                    extract_invokables(&original_impl.items, cpp_namespace_prefix)?;
+                                let mut extracted = extract_invokables(
+                                    &original_impl.items,
+                                    cpp_namespace_prefix,
+                                    &qt_ident,
+                                )?;
 
                                 object_invokables.append(&mut extracted.invokables);
                                 object_normal_methods.append(&mut extracted.normal_methods);
@@ -893,7 +849,7 @@ pub fn extract_qobject(
 
     // Read properties from the Data struct
     let object_properties = if let Some(ref original_struct) = original_data_struct {
-        extract_properties(original_struct, cpp_namespace_prefix)?
+        extract_properties(original_struct, cpp_namespace_prefix, &qt_ident)?
     } else {
         vec![]
     };
@@ -986,19 +942,19 @@ mod tests {
         assert_eq!(prop_first.ident.rust_ident.to_string(), "my_number");
         assert_eq!(prop_first.type_ident.idents.len(), 1);
         assert_eq!(prop_first.type_ident.idents[0].to_string(), "i32");
-        assert_eq!(prop_first.type_ident.is_ref, false);
+        assert!(!prop_first.type_ident.is_ref);
 
-        assert_eq!(prop_first.getter.is_some(), true);
+        assert!(prop_first.getter.is_some());
         let getter = prop_first.getter.as_ref().unwrap();
         assert_eq!(getter.cpp_ident.to_string(), "getMyNumber");
         assert_eq!(getter.rust_ident.to_string(), "my_number");
 
-        assert_eq!(prop_first.setter.is_some(), true);
+        assert!(prop_first.setter.is_some());
         let setter = prop_first.setter.as_ref().unwrap();
         assert_eq!(setter.cpp_ident.to_string(), "setMyNumber");
         assert_eq!(setter.rust_ident.to_string(), "set_my_number");
 
-        assert_eq!(prop_first.notify.is_some(), true);
+        assert!(prop_first.notify.is_some());
         let notify = prop_first.notify.as_ref().unwrap();
         assert_eq!(notify.cpp_ident.to_string(), "myNumberChanged");
         // TODO: does rust need a notify ident?
@@ -1064,13 +1020,13 @@ mod tests {
         // TODO: add extra checks when we read if this is a mut or not
         assert_eq!(param_first.type_ident.idents.len(), 1);
         assert_eq!(param_first.type_ident.idents[0].to_string(), "QString");
-        assert_eq!(param_first.type_ident.is_ref, true);
+        assert!(param_first.type_ident.is_ref);
 
         let param_second = &invokable.parameters[1];
         assert_eq!(param_second.ident.to_string(), "number");
         assert_eq!(param_second.type_ident.idents.len(), 1);
         assert_eq!(param_second.type_ident.idents[0].to_string(), "i32");
-        assert_eq!(param_second.type_ident.is_ref, false);
+        assert!(!param_second.type_ident.is_ref);
 
         // Check invokable ident
         let invokable_second = &qobject.invokables[1];
@@ -1104,19 +1060,19 @@ mod tests {
         assert_eq!(prop_first.ident.rust_ident.to_string(), "number");
         assert_eq!(prop_first.type_ident.idents.len(), 1);
         assert_eq!(prop_first.type_ident.idents[0].to_string(), "i32");
-        assert_eq!(prop_first.type_ident.is_ref, false);
+        assert!(!prop_first.type_ident.is_ref);
 
-        assert_eq!(prop_first.getter.is_some(), true);
+        assert!(prop_first.getter.is_some());
         let getter = prop_first.getter.as_ref().unwrap();
         assert_eq!(getter.cpp_ident.to_string(), "getNumber");
         assert_eq!(getter.rust_ident.to_string(), "number");
 
-        assert_eq!(prop_first.setter.is_some(), true);
+        assert!(prop_first.setter.is_some());
         let setter = prop_first.setter.as_ref().unwrap();
         assert_eq!(setter.cpp_ident.to_string(), "setNumber");
         assert_eq!(setter.rust_ident.to_string(), "set_number");
 
-        assert_eq!(prop_first.notify.is_some(), true);
+        assert!(prop_first.notify.is_some());
         let notify = prop_first.notify.as_ref().unwrap();
         assert_eq!(notify.cpp_ident.to_string(), "numberChanged");
         // TODO: does rust need a notify ident?
@@ -1128,19 +1084,19 @@ mod tests {
         assert_eq!(prop_second.ident.rust_ident.to_string(), "string");
         assert_eq!(prop_second.type_ident.idents.len(), 1);
         assert_eq!(prop_second.type_ident.idents[0].to_string(), "String");
-        assert_eq!(prop_second.type_ident.is_ref, false);
+        assert!(!prop_second.type_ident.is_ref);
 
-        assert_eq!(prop_second.getter.is_some(), true);
+        assert!(prop_second.getter.is_some());
         let getter = prop_second.getter.as_ref().unwrap();
         assert_eq!(getter.cpp_ident.to_string(), "getString");
         assert_eq!(getter.rust_ident.to_string(), "string");
 
-        assert_eq!(prop_second.setter.is_some(), true);
+        assert!(prop_second.setter.is_some());
         let setter = prop_second.setter.as_ref().unwrap();
         assert_eq!(setter.cpp_ident.to_string(), "setString");
         assert_eq!(setter.rust_ident.to_string(), "set_string");
 
-        assert_eq!(prop_second.notify.is_some(), true);
+        assert!(prop_second.notify.is_some());
         let notify = prop_second.notify.as_ref().unwrap();
         assert_eq!(notify.cpp_ident.to_string(), "stringChanged");
         // TODO: does rust need a notify ident?
@@ -1194,20 +1150,11 @@ mod tests {
         let param_first = &invokable.parameters[0];
         assert_eq!(param_first.ident.to_string(), "_cpp");
         assert_eq!(param_first.type_ident.idents.len(), 1);
-        assert_eq!(param_first.type_ident.idents[0].to_string(), "Pin");
-        assert_eq!(param_first.type_ident.is_ref, false);
-        if let QtTypes::Pin {
-            ident_namespace_str,
-            is_mut,
-            is_this,
-            type_idents,
-        } = &param_first.type_ident.qt_type
-        {
-            assert_eq!(ident_namespace_str, "CppObj");
-            assert_eq!(is_mut, &true);
-            assert_eq!(is_this, &true);
-            assert_eq!(type_idents.len(), 1);
-            assert_eq!(type_idents[0].to_string(), "CppObj");
+        assert_eq!(param_first.type_ident.idents[0].to_string(), "CppObj");
+        assert!(param_first.type_ident.is_ref);
+        assert!(param_first.type_ident.is_mut);
+        if let QtTypes::CppObj { external, .. } = &param_first.type_ident.qt_type {
+            assert_eq!(external, &false);
         } else {
             panic!();
         }
@@ -1217,13 +1164,13 @@ mod tests {
         // TODO: add extra checks when we read if this is a mut or not
         assert_eq!(param_second.type_ident.idents.len(), 1);
         assert_eq!(param_second.type_ident.idents[0].to_string(), "QString");
-        assert_eq!(param_second.type_ident.is_ref, true);
+        assert!(param_second.type_ident.is_ref);
 
         let param_third = &invokable.parameters[2];
         assert_eq!(param_third.ident.to_string(), "number");
         assert_eq!(param_third.type_ident.idents.len(), 1);
         assert_eq!(param_third.type_ident.idents[0].to_string(), "i32");
-        assert_eq!(param_third.type_ident.is_ref, false);
+        assert!(!param_third.type_ident.is_ref);
 
         // Check invokable ident
         let invokable_second = &qobject.invokables[1];
@@ -1236,20 +1183,11 @@ mod tests {
         let param_first = &invokable_second.parameters[0];
         assert_eq!(param_first.ident.to_string(), "_cpp");
         assert_eq!(param_first.type_ident.idents.len(), 1);
-        assert_eq!(param_first.type_ident.idents[0].to_string(), "Pin");
-        assert_eq!(param_first.type_ident.is_ref, false);
-        if let QtTypes::Pin {
-            ident_namespace_str,
-            is_mut,
-            is_this,
-            type_idents,
-        } = &param_first.type_ident.qt_type
-        {
-            assert_eq!(ident_namespace_str, "CppObj");
-            assert_eq!(is_mut, &true);
-            assert_eq!(is_this, &true);
-            assert_eq!(type_idents.len(), 1);
-            assert_eq!(type_idents[0].to_string(), "CppObj");
+        assert_eq!(param_first.type_ident.idents[0].to_string(), "CppObj");
+        assert!(param_first.type_ident.is_ref);
+        assert!(param_first.type_ident.is_mut);
+        if let QtTypes::CppObj { external, .. } = &param_first.type_ident.qt_type {
+            assert_eq!(external, &false);
         } else {
             panic!();
         }
