@@ -7,16 +7,12 @@ use convert_case::{Case, Casing};
 use proc_macro2::{Ident, TokenStream};
 use quote::{format_ident, quote, ToTokens};
 use std::collections::HashSet;
-use syn::spanned::Spanned;
-use syn::ImplItemMethod;
 
-use crate::extract::{ExtractMethodUnknownOptions, QObject, QtTypes};
-use crate::utils::{is_type_ident_ptr, type_to_namespace};
+use crate::extract::{Invokable, QObject, QtTypes};
+use crate::utils::type_to_namespace;
 
 /// A trait which we implement on QtTypes allowing retrieval of attributes of the enum value.
 trait RustType {
-    /// Whether this type is a Pin<T>
-    fn is_pin(&self) -> bool;
     /// Whether this type is a reference
     fn is_ref(&self) -> bool;
     /// Whether this type is a this (eg the T in Pin<T>)
@@ -28,19 +24,20 @@ trait RustType {
 }
 
 impl RustType for QtTypes {
-    /// Whether this type is a Pin<T>, this is used to find args to rename Pin to std::pin::Pin
-    fn is_pin(&self) -> bool {
-        match self {
-            Self::Pin { .. } => true,
-            _others => false,
-        }
-    }
-
     /// Whether this type should be a reference when used in Rust methods
     fn is_ref(&self) -> bool {
         match self {
-            Self::Pin { .. } => unreachable!(),
+            Self::Color => true,
+            Self::QDate => true,
+            Self::DateTime => true,
+            Self::QPoint => true,
             Self::QPointF => true,
+            Self::QRect => true,
+            Self::QRectF => true,
+            Self::QSize => true,
+            Self::QSizeF => true,
+            Self::QTime => true,
+            Self::Url => true,
             Self::Variant => true,
             Self::Str | Self::String => true,
             _others => false,
@@ -50,7 +47,7 @@ impl RustType for QtTypes {
     /// Whether this type is_this, this is used to determine if the type needs to be rewritten
     fn is_this(&self) -> bool {
         match self {
-            Self::Pin { is_this, .. } => is_this == &true,
+            Self::CppObj { external, .. } => external == &false,
             _others => false,
         }
     }
@@ -64,11 +61,18 @@ impl RustType for QtTypes {
             Self::I8 => format_ident!("i8"),
             Self::I16 => format_ident!("i16"),
             Self::I32 => format_ident!("i32"),
-            Self::Pin { .. } => unreachable!(),
-            // Pointer types do not use this function (TODO: yet?)
-            Self::Ptr { .. } => unreachable!(),
+            Self::Color | Self::QColor => format_ident!("QColor"),
+            Self::QDate => format_ident!("QDate"),
+            Self::QDateTime | Self::DateTime => format_ident!("QDateTime"),
+            Self::QPoint => format_ident!("QPoint"),
             Self::QPointF => format_ident!("QPointF"),
+            Self::QRect => format_ident!("QRect"),
+            Self::QRectF => format_ident!("QRectF"),
+            Self::QSize => format_ident!("QSize"),
+            Self::QSizeF => format_ident!("QSizeF"),
             Self::Str | Self::String | Self::QString => format_ident!("QString"),
+            Self::QTime => format_ident!("QTime"),
+            Self::QUrl | Self::Url => format_ident!("QUrl"),
             Self::QVariant | Self::Variant => format_ident!("QVariant"),
             Self::U8 => format_ident!("u8"),
             Self::U16 => format_ident!("u16"),
@@ -81,18 +85,30 @@ impl RustType for QtTypes {
     fn full_param_type(&self) -> TokenStream {
         match self {
             Self::Bool => quote! {bool},
+            Self::CppObj {
+                external,
+                combined_name,
+                ..
+            } if external == &true => {
+                quote! {cxx::UniquePtr<ffi::#combined_name>}
+            }
             Self::F32 => quote! {f32},
             Self::F64 => quote! {f64},
             Self::I8 => quote! {i8},
             Self::I16 => quote! {i16},
             Self::I32 => quote! {i32},
-            Self::Pin { .. } => unreachable!(),
-            Self::Ptr { ident_str, .. } => {
-                let ident = format_ident!("{}", ident_str);
-                quote! {cxx::UniquePtr<ffi::#ident>}
-            }
+            Self::Color | Self::QColor => quote! {cxx_qt_lib::QColor},
+            Self::QDate => quote! {cxx_qt_lib::QDate},
+            Self::QDateTime | Self::DateTime => quote! {cxx_qt_lib::QDateTime},
+            Self::QPoint => quote! {cxx_qt_lib::QPoint},
             Self::QPointF => quote! {cxx_qt_lib::QPointF},
+            Self::QRect => quote! {cxx_qt_lib::QRect},
+            Self::QRectF => quote! {cxx_qt_lib::QRectF},
+            Self::QSize => quote! {cxx_qt_lib::QSize},
+            Self::QSizeF => quote! {cxx_qt_lib::QSizeF},
+            Self::QTime => quote! {cxx_qt_lib::QTime},
             Self::Str | Self::String | Self::QString => quote! {cxx_qt_lib::QString},
+            Self::QUrl | Self::Url => quote! {cxx_qt_lib::QUrl},
             Self::QVariant | Self::Variant => quote! {cxx_qt_lib::QVariant},
             Self::U8 => quote! {u8},
             Self::U16 => quote! {u16},
@@ -160,20 +176,20 @@ pub fn generate_qobject_cxx(
 
         Ok(())
     };
-    // Closure which retrieves Object from crate::module::Object and swaps to crate::module::CppObj
+    // Closure which retrieves Object from crate::module::Object and swaps to crate::module::FFICppObj
     let type_idents_to_ptr_class_name_and_ffi_type = |type_idents: &Vec<Ident>| {
         // Build the class name of the pointer, eg Object in crate::module::Object
         //
         // We can assume that unwrap will work here as we have checked that type_idents is not empty
         let ptr_class_name = type_idents.last().unwrap().clone();
 
-        // Swap the last type segment to be CppObj
-        // so that crate::module::Object becomes crate::module::CppObj
+        // Swap the last type segment to be FFICppObj
+        // so that crate::module::Object becomes crate::module::FFICppObj
         //
         // As we will generate a public type which points to the ffi type at the module level
         let mut type_idents_ffi = type_idents.clone();
         type_idents_ffi.pop();
-        type_idents_ffi.push(format_ident!("CppObj"));
+        type_idents_ffi.push(format_ident!("FFICppObj"));
 
         (ptr_class_name, type_idents_ffi)
     };
@@ -184,8 +200,14 @@ pub fn generate_qobject_cxx(
     // to a custom name for C++ or Rust side?
     for i in &obj.invokables {
         // Cache the ident and parameters as they are used multiple times later
-        let ident = &i.ident.rust_ident;
-        let ident_cpp_str = &i.ident.cpp_ident.to_string();
+        let (ident, ident_cpp_str) = if let Some(ident_wrapper) = &i.ident_wrapper {
+            (
+                &ident_wrapper.rust_ident,
+                ident_wrapper.cpp_ident.to_string(),
+            )
+        } else {
+            (&i.ident.rust_ident, i.ident.cpp_ident.to_string())
+        };
         let parameters = &i.parameters;
 
         // TODO: invokables need to also become freestanding functions that
@@ -193,7 +215,7 @@ pub fn generate_qobject_cxx(
         // inside a wrapper. The functions that are impl'ed on the Rs class
         // will then simply create the wrapper and call the free functions.
         //
-        // As a first step we could maybe just add a `cpp: Pin<&mut CppObj>`
+        // As a first step we could maybe just add a `cpp: Pin<&mut FFICppObj>`
         // argument to invokables so that users can manually wrap it.
 
         // Determine if the invokable has any parameter
@@ -204,7 +226,13 @@ pub fn generate_qobject_cxx(
                 let type_idents = &return_type.idents;
 
                 // Determine if the return type is a ref
-                if return_type.is_ref {
+                if return_type.qt_type.is_opaque() {
+                    let return_type_ident = return_type.qt_type.param_type_ident();
+                    rs_functions.push(quote! {
+                        #[cxx_name = #ident_cpp_str]
+                        fn #ident(self: &#rust_class_name) -> UniquePtr<#return_type_ident>;
+                    });
+                } else if return_type.is_ref {
                     rs_functions.push(quote! {
                         #[cxx_name = #ident_cpp_str]
                         fn #ident(self: &#rust_class_name) -> &#(#type_idents)::*;
@@ -230,41 +258,37 @@ pub fn generate_qobject_cxx(
 
                 // If the type is Pin<T> then we need to change extract differently
                 match &p.type_ident.qt_type {
-                    QtTypes::Pin {
-                        is_mut,
-                        is_this,
-                        type_idents,
+                    QtTypes::CppObj {
+                        external,
+                        rust_type_idents,
+                        combined_name,
                         ..
                     } => {
-                        // If the Pin<T> is to our own type, then we refer to the C++ class name
-                        // not the external crate (when it is a sub object)
-                        if *is_this {
-                            if *is_mut {
-                                parameters_quotes.push(quote! { #ident: Pin<&mut #class_name> });
-                            } else {
-                                parameters_quotes.push(quote! { #ident: Pin<&#class_name> });
-                            }
-                        } else {
-                            // Retrieve Object from crate::module::Object and swap to crate::module::CppObj
-                            let (ptr_class_name, type_idents_ffi) =
-                                type_idents_to_ptr_class_name_and_ffi_type(type_idents);
+                        if *external {
+                            // TODO: these two will likely replace type_idents_to_ptr_class_name_and_ffi_type
+                            // once we remove Ptr as a type
+                            let rust_type_idents = rust_type_idents
+                                .iter()
+                                .take(rust_type_idents.len() - 1)
+                                .cloned()
+                                .chain(vec![format_ident!("FFICppObj")])
+                                .collect::<Vec<Ident>>();
 
-                            // Add type definition for the struct name we are a Pin for to the Rust bridge
-                            //
-                            // Ensure that we only do this once
+                            parameters_quotes.push(quote! {
+                                #ident: Pin<&mut #combined_name>
+                            });
+
+                            // Add the type of the external object to the C++ bridge
                             cpp_types_push_unique(
                                 &mut cpp_functions,
                                 &mut cpp_types,
-                                &ptr_class_name,
-                                type_idents_ffi,
+                                combined_name,
+                                rust_type_idents,
                             )?;
-
-                            if *is_mut {
-                                parameters_quotes
-                                    .push(quote! { #ident: Pin<&mut #ptr_class_name> });
-                            } else {
-                                parameters_quotes.push(quote! { #ident: Pin<&#ptr_class_name> });
-                            }
+                        } else {
+                            parameters_quotes.push(quote! {
+                                #ident: Pin<&mut #(#rust_type_idents)::*>
+                            });
                         }
                     }
                     _others => {
@@ -290,7 +314,13 @@ pub fn generate_qobject_cxx(
                 let type_idents = &return_type.idents;
 
                 // Determine if the return type is a ref
-                if return_type.is_ref {
+                if return_type.qt_type.is_opaque() {
+                    let return_type_ident = return_type.qt_type.param_type_ident();
+                    rs_functions.push(quote! {
+                        #[cxx_name = #ident_cpp_str]
+                        fn #ident(self: &#rust_class_name, #(#parameters_quotes),*) -> UniquePtr<#return_type_ident>;
+                    });
+                } else if return_type.is_ref {
                     rs_functions.push(quote! {
                         #[cxx_name = #ident_cpp_str]
                         fn #ident(self: &#rust_class_name, #(#parameters_quotes),*) -> &#(#type_idents)::*;
@@ -319,8 +349,8 @@ pub fn generate_qobject_cxx(
         let property_ident_snake = property.ident.rust_ident.to_string().to_case(Case::Snake);
         let property_ident_pascal = property.ident.rust_ident.to_string().to_case(Case::Pascal);
 
-        // This type is a pointer, so special case the C++ functions and no Rust functions
-        if is_type_ident_ptr(type_idents) {
+        // This type is a pointer (CppObj), so special case the C++ functions and no Rust functions
+        if let QtTypes::CppObj { combined_name, .. } = &property.type_ident.qt_type {
             // Check that type_idents is not empty
             if type_idents.is_empty() {
                 return Err(syn::Error::new(
@@ -330,9 +360,8 @@ pub fn generate_qobject_cxx(
                 .to_compile_error());
             }
 
-            // Retrieve Object from crate::module::Object and swap to crate::module::CppObj
-            let (ptr_class_name, type_idents_ffi) =
-                type_idents_to_ptr_class_name_and_ffi_type(type_idents);
+            // Retrieve Object from crate::module::Object and swap to crate::module::FFICppObj
+            let (_, type_idents_ffi) = type_idents_to_ptr_class_name_and_ffi_type(type_idents);
 
             // Add type definition for the class name we are a pointer for to the C++ bridge
             //
@@ -340,7 +369,7 @@ pub fn generate_qobject_cxx(
             cpp_types_push_unique(
                 &mut cpp_functions,
                 &mut cpp_types,
-                &ptr_class_name,
+                combined_name,
                 type_idents_ffi,
             )?;
 
@@ -353,9 +382,9 @@ pub fn generate_qobject_cxx(
             // Add the getter and setter to C++ bridge
             cpp_functions.push(quote! {
                 #[rust_name = #getter_str]
-                fn #getter_cpp(self: Pin<&mut #class_name>) -> UniquePtr<#ptr_class_name>;
+                fn #getter_cpp(self: Pin<&mut #class_name>) -> UniquePtr<#combined_name>;
                 #[rust_name = #setter_str]
-                fn #setter_cpp(self: Pin<&mut #class_name>, value: UniquePtr<#ptr_class_name>);
+                fn #setter_cpp(self: Pin<&mut #class_name>, value: UniquePtr<#combined_name>);
             });
         // This is a normal primitive type so add Rust getters and setters
         } else {
@@ -426,16 +455,33 @@ pub fn generate_qobject_cxx(
 
                 type #class_name;
 
+                #[namespace = ""]
+                type QColor = cxx_qt_lib::QColor;
+                #[namespace = ""]
+                type QDate = cxx_qt_lib::QDate;
+                #[namespace = ""]
+                type QDateTime = cxx_qt_lib::QDateTime;
+                #[namespace = ""]
+                type QPoint = cxx_qt_lib::QPoint;
                 #[namespace = ""]
                 type QPointF = cxx_qt_lib::QPointF;
                 #[namespace = ""]
+                type QRect = cxx_qt_lib::QRect;
+                #[namespace = ""]
+                type QRectF = cxx_qt_lib::QRectF;
+                #[namespace = ""]
+                type QSize = cxx_qt_lib::QSize;
+                #[namespace = ""]
+                type QSizeF = cxx_qt_lib::QSizeF;
+                #[namespace = ""]
                 type QString = cxx_qt_lib::QString;
                 #[namespace = ""]
+                type QTime = cxx_qt_lib::QTime;
+                #[namespace = ""]
+                type QUrl = cxx_qt_lib::QUrl;
+                #[namespace = ""]
                 type QVariant = cxx_qt_lib::QVariant;
 
-                #[namespace = "CxxQt"]
-                type Variant = cxx_qt_lib::Variant;
-
                 #(#cpp_functions)*
 
                 #[rust_name = "new_cpp_object"]
@@ -458,7 +504,7 @@ pub fn generate_qobject_cxx(
             }
         }
 
-        pub type CppObj = ffi::#class_name;
+        pub type FFICppObj = ffi::#class_name;
         pub type Property = ffi::Property;
     };
 
@@ -471,8 +517,8 @@ fn generate_cpp_object_initialiser(obj: &QObject) -> TokenStream {
 
     // We assume that all Data classes implement default
     let output = quote! {
-        fn initialise_cpp(cpp: std::pin::Pin<&mut CppObj>) {
-            let mut wrapper = CppObjWrapper::new(cpp);
+        fn initialise_cpp(cpp: std::pin::Pin<&mut FFICppObj>) {
+            let mut wrapper = CppObj::new(cpp);
             wrapper.grab_values_from_data(&#data_class_name::default());
         }
     };
@@ -511,7 +557,7 @@ fn generate_property_methods_rs(obj: &QObject) -> Result<Vec<TokenStream>, Token
         let cpp_getter_ident = &property.getter.as_ref().unwrap().rust_ident;
         let cpp_setter_ident = &property.setter.as_ref().unwrap().rust_ident;
 
-        if is_type_ident_ptr(&property.type_ident.idents) {
+        if let QtTypes::CppObj { .. } = property.type_ident.qt_type {
             let ident = &property.ident.rust_ident;
             let take_ident = format_ident!("take_{}", ident);
             let give_ident = format_ident!("give_{}", ident);
@@ -555,52 +601,6 @@ fn generate_property_methods_rs(obj: &QObject) -> Result<Vec<TokenStream>, Token
     Ok(property_methods)
 }
 
-fn is_field_ptr(field: &syn::Field) -> bool {
-    // Determine the type of the field
-    let ty_path;
-    match &field.ty {
-        syn::Type::Path(path) => {
-            ty_path = path;
-        }
-        syn::Type::Reference(syn::TypeReference { elem, .. }) => {
-            if let syn::Type::Path(path) = &**elem {
-                ty_path = path;
-            } else {
-                // Unknown type, just ignore so we pass through
-                return true;
-            }
-        }
-        _others => {
-            // Unknown type, just ignore so we pass through
-            return true;
-        }
-    }
-
-    // Filter any fields that have a type which is a pointer
-    !is_type_ident_ptr(
-        &ty_path
-            .path
-            .segments
-            .iter()
-            .map(|segment| segment.ident.to_owned())
-            .collect::<Vec<syn::Ident>>(),
-    )
-}
-
-/// Return the fields of the struct filtered by the given closure
-fn filter_fields<F: Fn(&syn::Field) -> bool>(
-    original_struct: &syn::ItemStruct,
-    filter_closure: F,
-) -> Vec<&syn::Field> {
-    // Filter the fields of the struct to remove any pointer fields
-    // as they are instead stored in the C++ object and not owned by the rust side
-    original_struct
-        .fields
-        .iter()
-        .filter(|field| filter_closure(field))
-        .collect::<Vec<&syn::Field>>()
-}
-
 /// Builds a struct with th given new fields
 fn build_struct_with_fields(
     original_struct: &syn::ItemStruct,
@@ -630,179 +630,82 @@ fn build_struct_with_fields(
     }
 }
 
-/// Add std::pin to Pin arguments in a method
-fn fix_pin_path(method: &mut ImplItemMethod, pin_args: &[usize]) -> Result<(), TokenStream> {
-    // Rewrite args
-    method
-        .sig
-        .inputs
-        .iter_mut()
-        // We can skip self as that's not in our parameters above
-        //
-        // TODO: note this assumes that the first argument is self
-        .skip(1)
-        // We only want to rewrite pinned args
-        .enumerate()
-        .filter(|(index, _)| pin_args.contains(index))
-        // Rewrite the type
-        //
-        // We add std::pin to Pin, and swap the last type inside a Pin<&a::T> from T to CppObj
-        .map(|(_, item)| {
-            if let syn::FnArg::Typed(syn::PatType { ref mut ty, .. }) = item {
-                let ty_path;
+/// Generate the wrapper method for a given invokable
+fn invokable_generate_wrapper(
+    invokable: &Invokable,
+    ident_wrapper: &Ident,
+) -> Result<TokenStream, TokenStream> {
+    let ident = &invokable.ident.rust_ident;
+    let return_type_orig = invokable.original_method.sig.output.clone();
 
-                match ty.as_mut() {
-                    syn::Type::Path(ref mut path) => {
-                        ty_path = path;
-                    }
-                    // TODO: do we support TypeReference for &Pin<T>?
-                    // Type::Reference(TypeReference { elem, .. }) => {
-                    //     // If the type is a path then extract it and mark is_ref
-                    //     if let Type::Path(path) = &**elem {
-                    //         ty_path = path;
-                    //     }
-                    // }
-                    //
-                    _others => {
-                        return Err(syn::Error::new(
-                            item.span(),
-                            "Pin<T> argument must be a path, we do not support reference yet",
-                        )
-                        .to_compile_error())
-                    }
-                }
+    let mut input_parameters = vec![];
+    let mut output_parameters = vec![];
+    let mut wrappers = vec![];
 
-                // Add std::pin to Pin
-                ty_path.path.segments.insert(0, format_ident!("std").into());
-                ty_path.path.segments.insert(1, format_ident!("pin").into());
-
-                // From a::Pin<&b::T> we want Pin<&b::T> (we ultimately want to get to T)
-                if let Some(segment) = ty_path.path.segments.last_mut() {
-                    // From Pin<&b::T> we want &b::T
-                    if let syn::PathArguments::AngleBracketed(arguments) = &mut segment.arguments {
-                        // From &b::T we want b::T
-                        //
-                        // TODO: do we need to support non reference? or will it always be reference?
-                        if let Some(syn::GenericArgument::Type(syn::Type::Reference(
-                            syn::TypeReference { elem, .. },
-                        ))) = arguments.args.first_mut()
-                        {
-                            if let syn::Type::Path(arg_path) = &mut **elem {
-                                // From b::T we want T
-                                if let Some(segment) = arg_path.path.segments.last_mut() {
-                                    // Always set last type in Pin<T> to CppObj
-                                    segment.ident = format_ident!("CppObj");
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-            Ok(item)
-        })
-        .collect::<Result<Vec<_>, TokenStream>>()?;
-
-    Ok(())
-}
-
-/// Add std::pin to Pin and cxx_qt_lib::qstring to QString arguments for all impl methods
-fn fix_impl_methods(impl_: &syn::ItemImpl) -> Result<TokenStream, TokenStream> {
-    let mut cloned = impl_.to_owned();
-
-    for mut item in &mut cloned.items {
-        if let syn::ImplItem::Method(m) = &mut item {
-            *m = fix_method_params(m, ExtractMethodUnknownOptions::ParseAllTypes)?;
+    for param in &invokable.parameters {
+        let param_ident = &param.ident;
+        let is_mut = if param.type_ident.is_mut {
+            quote! { mut }
         } else {
-            return Err(
-                syn::Error::new(item.span(), "Only methods are supported.").to_compile_error()
-            );
+            quote! {}
+        };
+        let is_ref = if param.type_ident.is_ref {
+            quote! { & }
+        } else {
+            quote! {}
+        };
+
+        if let QtTypes::CppObj {
+            rust_type_idents, ..
+        } = &param.type_ident.qt_type
+        {
+            // Create Rust idents with CppObj and FFICppObj at the end
+            let rust_idents_module = rust_type_idents
+                .iter()
+                .take(rust_type_idents.len() - 1)
+                .cloned()
+                .collect::<Vec<Ident>>();
+            let rust_idents_ffi = rust_idents_module
+                .iter()
+                .cloned()
+                .chain(vec![format_ident!("FFICppObj")]);
+            let rust_idents_cpp_obj = rust_idents_module
+                .iter()
+                .cloned()
+                .chain(vec![format_ident!("CppObj")]);
+
+            input_parameters
+                .push(quote! { #param_ident: std::pin::Pin<&mut #(#rust_idents_ffi)::*> });
+
+            wrappers.push(quote! {
+                let mut #param_ident = #(#rust_idents_cpp_obj)::*::new(#param_ident);
+            });
+            output_parameters.push(quote! { #is_ref #is_mut #param_ident });
+        } else {
+            let param_type = &param.type_ident.idents;
+            input_parameters.push(quote! { #param_ident: #is_ref #is_mut #(#param_type)::* });
+            output_parameters.push(quote! { #param_ident });
         }
     }
 
-    Ok(quote! { #cloned })
-}
-
-/// Add std::pin to Pin and cxx_qt_lib::qstring to QString arguments
-fn fix_method_params(
-    method: &ImplItemMethod,
-    extract_options: ExtractMethodUnknownOptions,
-) -> Result<ImplItemMethod, TokenStream> {
-    let mut method = method.clone();
-
-    // Extract parameters from the method
-    let parameters = crate::extract::extract_method_params(&method, &[""], extract_options)?;
-
-    // Find which arguments are using Pin<T>
-    let pin_args = parameters
-        .iter()
-        .enumerate()
-        .filter(|(_, parameter)| parameter.type_ident.qt_type.is_pin())
-        .map(|(index, _)| index)
-        .collect::<Vec<usize>>();
-
-    let qstring_args = parameters
-        .iter()
-        .enumerate()
-        .filter(|(_, parameter)| parameter.type_ident.qt_type == QtTypes::QString)
-        .map(|(index, _)| index)
-        .collect::<Vec<usize>>();
-
-    if !pin_args.is_empty() {
-        fix_pin_path(&mut method, &pin_args)?;
-    }
-
-    if !qstring_args.is_empty() {
-        // Rewrite args
-        method
-            .sig
-            .inputs
-            .iter_mut()
-            // We can skip self as that's not in our parameters above
-            //
-            // TODO: note this assumes that the first argument is self
-            .skip(1)
-            .enumerate()
-            .filter(|(index, _)| qstring_args.contains(index))
-            // Rewrite the type
-            //
-            // We add cxx_qt_lib to QString
-            .map(|(_, item)| {
-                if let syn::FnArg::Typed(syn::PatType { ref mut ty, .. }) = item {
-                    let ty_path;
-
-                    match ty.as_mut() {
-                        syn::Type::Reference(syn::TypeReference { elem, .. }) => {
-                            // If the type is a path then extract it and mark is_ref
-                            if let syn::Type::Path(ref mut path) = &mut **elem {
-                                ty_path = path;
-                            } else {
-                                return Err(syn::Error::new(
-                                    item.span(),
-                                    "QString args must be valid references.",
-                                )
-                                .to_compile_error());
-                            }
-                        }
-                        _others => {
-                            return Err(syn::Error::new(
-                                item.span(),
-                                "QString args must be references.",
-                            )
-                            .to_compile_error())
-                        }
-                    }
-
-                    ty_path
-                        .path
-                        .segments
-                        .insert(0, format_ident!("cxx_qt_lib").into());
+    if let Some(return_type) = &invokable.return_type {
+        if return_type.qt_type.is_opaque() {
+            let return_type_ident = return_type.qt_type.full_param_type();
+            return Ok(quote! {
+                fn #ident_wrapper(&self, #(#input_parameters),*) -> cxx::UniquePtr<#return_type_ident> {
+                    #(#wrappers)*
+                    return self.#ident(#(#output_parameters),*).to_unique_ptr();
                 }
-                Ok(item)
-            })
-            .collect::<Result<Vec<_>, TokenStream>>()?;
+            });
+        }
     }
 
-    Ok(method)
+    Ok(quote! {
+        fn #ident_wrapper(&self, #(#input_parameters),*) #return_type_orig {
+            #(#wrappers)*
+            return self.#ident(#(#output_parameters),*);
+        }
+    })
 }
 
 /// Generate all the Rust code required to communicate with a QObject backed by generated C++ code
@@ -840,7 +743,7 @@ pub fn generate_qobject_rs(
 
     // Cache the rust class name
     let rust_class_name = format_ident!("RustObj");
-    let rust_wrapper_name = format_ident!("CppObjWrapper");
+    let rust_wrapper_name = format_ident!("CppObj");
 
     // Generate cxx block
     let cxx_block = generate_qobject_cxx(obj, cpp_namespace_prefix)?;
@@ -851,11 +754,22 @@ pub fn generate_qobject_rs(
     // do we need to rewrite the field to their data struct?
     let data_struct_name = &obj.original_data_struct.ident;
     // Build a list of the fields that aren't pointers as they are stored on C++ side
-    let data_fields_no_ptr = filter_fields(&obj.original_data_struct, is_field_ptr);
+    let data_fields_no_ptr = obj
+        .properties
+        .iter()
+        .zip(&obj.original_data_struct.fields)
+        .filter_map(|(prop, field)| {
+            if let QtTypes::CppObj { .. } = prop.type_ident.qt_type {
+                None
+            } else {
+                Some(field)
+            }
+        })
+        .collect::<Vec<&syn::Field>>();
     // TODO: we need to update this to only store fields defined as "private" once we have an API for that
     let data_struct = build_struct_with_fields(&obj.original_data_struct, &data_fields_no_ptr);
 
-    // Build a converter for Data -> CppObjWrapper
+    // Build a converter for Data -> CppObj
     let data_struct_impl = {
         let mut fields_into = vec![];
         // If there are no filtered fields then use _value
@@ -883,6 +797,12 @@ pub fn generate_qobject_rs(
                     }
                 }
             }
+
+            impl<'a> From<&mut #rust_wrapper_name<'a>> for #data_struct_name {
+                fn from(#value_ident: &mut #rust_wrapper_name<'a>) -> Self {
+                    Self::from(&*#value_ident)
+                }
+            }
         }
     };
 
@@ -890,22 +810,22 @@ pub fn generate_qobject_rs(
     let property_methods = generate_property_methods_rs(obj)?;
 
     // Capture methods, trait impls, use decls so they can used by quote
+    let invokable_method_wrappers = obj
+        .invokables
+        .iter()
+        .filter_map(|i| {
+            i.ident_wrapper
+                .as_ref()
+                .map(|ident_wrapper| invokable_generate_wrapper(i, &ident_wrapper.rust_ident))
+        })
+        .collect::<Result<Vec<TokenStream>, TokenStream>>()?;
+
     let invokable_methods = obj
         .invokables
         .iter()
-        .map(|m| {
-            fix_method_params(
-                &m.original_method,
-                ExtractMethodUnknownOptions::ParseAllTypes,
-            )
-        })
-        .collect::<Result<Vec<syn::ImplItemMethod>, TokenStream>>()?;
-
-    let normal_methods = obj
-        .normal_methods
-        .iter()
-        .map(|m| fix_method_params(m, ExtractMethodUnknownOptions::IgnoreUnknownTypes))
-        .collect::<Result<Vec<syn::ImplItemMethod>, TokenStream>>()?;
+        .map(|m| m.original_method.clone())
+        .collect::<Vec<syn::ImplItemMethod>>();
+    let normal_methods = &obj.normal_methods;
 
     let original_trait_impls = &obj.original_trait_impls;
     let original_passthrough_decls = &obj.original_passthrough_decls;
@@ -914,18 +834,20 @@ pub fn generate_qobject_rs(
     let initialiser_fn = generate_cpp_object_initialiser(obj);
 
     // Build our filtered rust struct
-    //
-    // TODO: once fields are stored on this C++ side, this can change
     let rust_struct = build_struct_with_fields(
         &obj.original_rust_struct,
-        &filter_fields(&obj.original_rust_struct, is_field_ptr),
+        &obj.original_rust_struct
+            .fields
+            .iter()
+            .collect::<Vec<&syn::Field>>(),
     );
 
     // Define a function to handle update requests if we have one
     let handle_update_request = if obj.handle_updates_impl.is_some() {
         quote! {
-            fn call_handle_update_request(&mut self, cpp: std::pin::Pin<&mut CppObj>) {
-                self.handle_update_request(cpp);
+            fn call_handle_update_request(&mut self, cpp: std::pin::Pin<&mut FFICppObj>) {
+                let mut cpp = CppObj::new(cpp);
+                self.handle_update_request(&mut cpp);
             }
         }
     } else {
@@ -935,8 +857,9 @@ pub fn generate_qobject_rs(
     // Define a function to handle property changes if we have one
     let handle_property_change = if obj.handle_property_change_impl.is_some() {
         quote! {
-            fn call_handle_property_change(&mut self, cpp: std::pin::Pin<&mut CppObj>, property: Property) {
-                self.handle_property_change(cpp, property);
+            fn call_handle_property_change(&mut self, cpp: std::pin::Pin<&mut FFICppObj>, property: Property) {
+                let mut cpp = CppObj::new(cpp);
+                self.handle_property_change(&mut cpp, property);
             }
         }
     } else {
@@ -945,6 +868,7 @@ pub fn generate_qobject_rs(
 
     let rust_struct_impl = quote! {
         impl #rust_class_name {
+            #(#invokable_method_wrappers)*
             #(#invokable_methods)*
             #(#normal_methods)*
 
@@ -956,7 +880,7 @@ pub fn generate_qobject_rs(
     // Create a struct that wraps the CppObject with a nicer interface
     let wrapper_struct = quote! {
         pub struct #rust_wrapper_name<'a> {
-            cpp: std::pin::Pin<&'a mut CppObj>,
+            cpp: std::pin::Pin<&'a mut FFICppObj>,
         }
     };
 
@@ -986,7 +910,7 @@ pub fn generate_qobject_rs(
 
     let wrapper_struct_impl = quote! {
         impl<'a> #rust_wrapper_name<'a> {
-            fn new(cpp: std::pin::Pin<&'a mut CppObj>) -> Self {
+            pub fn new(cpp: std::pin::Pin<&'a mut FFICppObj>) -> Self {
                 Self { cpp }
             }
 
@@ -995,7 +919,7 @@ pub fn generate_qobject_rs(
             pub fn update_requester(&self) -> cxx_qt_lib::update_requester::UpdateRequester {
                 use cxx_qt_lib::update_requester::{CxxQObject, UpdateRequester};
 
-                let ptr: *const CppObj = unsafe { &*self.cpp.as_ref() };
+                let ptr: *const FFICppObj = unsafe { &*self.cpp.as_ref() };
                 unsafe { UpdateRequester::new(ptr as *mut CxxQObject) }
             }
 
@@ -1014,18 +938,11 @@ pub fn generate_qobject_rs(
     if obj.handle_property_change_impl.is_some() {
         use_traits.push(quote! { use cxx_qt_lib::PropertyChangeHandler; });
     }
+    // TODO: only push if we have an opaque param or return type?
+    use_traits.push(quote! { use cxx_qt_lib::ToUniquePtr; });
 
-    let handle_updates_impl = if let Some(impl_) = &obj.handle_updates_impl {
-        fix_impl_methods(impl_)?
-    } else {
-        quote! {}
-    };
-
-    let handle_property_change_impl = if let Some(impl_) = &obj.handle_property_change_impl {
-        fix_impl_methods(impl_)?
-    } else {
-        quote! {}
-    };
+    let handle_updates_impl = &obj.handle_updates_impl;
+    let handle_property_change_impl = &obj.handle_property_change_impl;
 
     // Build our rewritten module that replaces the input from the macro
     let output = quote! {
