@@ -9,7 +9,7 @@ mod my_object {
     struct RustObj;
 
     impl PropertyChangeHandler<CppObj, Property> for RustObj {
-        fn handle_property_change(&mut self, _cpp: Pin<&mut CppObj>, _property: Property) {
+        fn handle_property_change(&mut self, _cpp: &mut CppObj, _property: Property) {
             println!("change")
         }
     }
