@@ -1,10 +1,12 @@
 mod my_object {
+    use cxx_qt_lib::QString;
+
     #[derive(Default)]
     struct RustObj;
 
     impl RustObj {
         #[invokable]
-        fn say_hi(&self, _cpp: Pin<&mut CppObj>, string: &QString, number: i32) {
+        fn say_hi(&self, _cpp: &mut CppObj, string: &QString, number: i32) {
             println!(
                 "Hi from Rust! String is {} and number is {}",
                 string, number
@@ -12,7 +14,7 @@ mod my_object {
         }
 
         #[invokable]
-        fn say_bye(&self, _cpp: Pin<&mut CppObj>) {
+        fn say_bye(&self, _cpp: &mut CppObj) {
             println!("Bye from Rust!");
         }
     }
