@@ -1,4 +1,6 @@
 mod my_object {
+    use cxx_qt_lib::QString;
+
     #[derive(Default)]
     struct RustObj;
 
@@ -18,7 +20,7 @@ mod my_object {
     }
 
     impl UpdateRequestHandler<CppObj> for RustObj {
-        fn handle_update_request(&mut self, _cpp: Pin<&mut CppObj>) {
+        fn handle_update_request(&mut self, _cpp: &mut CppObj) {
             println!("update")
         }
     }
