@@ -30,8 +30,12 @@ MyObject::setNumber(qint32 value)
   if (value != m_number) {
     m_number = value;
 
-    Q_EMIT numberChanged();
-    m_rustObj->handlePropertyChange(*this, Property::Number);
+    runOnGUIThread([&]() { Q_EMIT numberChanged(); });
+
+    runOnGUIThread([&]() {
+      const std::lock_guard<std::mutex> guard(m_rustObjMutex);
+      m_rustObj->handlePropertyChange(*this, Property::Number);
+    });
   }
 }
 
@@ -52,15 +56,19 @@ MyObject::setString(const QString& value)
   if (value != m_string) {
     m_string = value;
 
-    Q_EMIT stringChanged();
-    m_rustObj->handlePropertyChange(*this, Property::String);
+    runOnGUIThread([&]() { Q_EMIT stringChanged(); });
+
+    runOnGUIThread([&]() {
+      const std::lock_guard<std::mutex> guard(m_rustObjMutex);
+      m_rustObj->handlePropertyChange(*this, Property::String);
+    });
   }
 }
 
-std::unique_ptr<MyObject>
+std::unique_ptr<CppObj>
 newCppObject()
 {
-  return std::make_unique<MyObject>();
+  return std::make_unique<CppObj>();
 }
 
 } // namespace cxx_qt::my_object
