@@ -16,19 +16,21 @@ MyObject::~MyObject() = default;
 void
 MyObject::sayHi(const QString& string, qint32 number)
 {
+  const std::lock_guard<std::mutex> guard(m_rustObjMutex);
   m_rustObj->sayHi(string, number);
 }
 
 void
 MyObject::sayBye()
 {
+  const std::lock_guard<std::mutex> guard(m_rustObjMutex);
   m_rustObj->sayBye();
 }
 
-std::unique_ptr<MyObject>
+std::unique_ptr<CppObj>
 newCppObject()
 {
-  return std::make_unique<MyObject>();
+  return std::make_unique<CppObj>();
 }
 
 } // namespace cxx_qt::my_object
