@@ -16,25 +16,28 @@ MyObject::~MyObject() = default;
 qint32
 MyObject::doubleNumber(qint32 number)
 {
+  const std::lock_guard<std::mutex> guard(m_rustObjMutex);
   return m_rustObj->doubleNumber(number);
 }
 
 QString
 MyObject::helloMessage(const QString& msg)
 {
-  return rustStringToQString(m_rustObj->helloMessage(msg));
+  const std::lock_guard<std::mutex> guard(m_rustObjMutex);
+  return std::move(*m_rustObj->helloMessageWrapper(msg));
 }
 
 QString
 MyObject::staticMessage()
 {
-  return rustStrToQString(m_rustObj->staticMessage());
+  const std::lock_guard<std::mutex> guard(m_rustObjMutex);
+  return std::move(*m_rustObj->staticMessageWrapper());
 }
 
-std::unique_ptr<MyObject>
+std::unique_ptr<CppObj>
 newCppObject()
 {
-  return std::make_unique<MyObject>();
+  return std::make_unique<CppObj>();
 }
 
 } // namespace cxx_qt::my_object
