@@ -30,7 +30,7 @@ MyObject::setNumber(qint32 value)
   if (value != m_number) {
     m_number = value;
 
-    Q_EMIT numberChanged();
+    runOnGUIThread([&]() { Q_EMIT numberChanged(); });
   }
 }
 
@@ -51,14 +51,14 @@ MyObject::setString(const QString& value)
   if (value != m_string) {
     m_string = value;
 
-    Q_EMIT stringChanged();
+    runOnGUIThread([&]() { Q_EMIT stringChanged(); });
   }
 }
 
-std::unique_ptr<MyObject>
+std::unique_ptr<CppObj>
 newCppObject()
 {
-  return std::make_unique<MyObject>();
+  return std::make_unique<CppObj>();
 }
 
 } // namespace cxx_qt::my_object
