@@ -16,19 +16,21 @@ MyObject::~MyObject() = default;
 void
 MyObject::sayHi(const QString& string, qint32 number)
 {
-  m_rustObj->sayHi(*this, string, number);
+  const std::lock_guard<std::mutex> guard(m_rustObjMutex);
+  m_rustObj->sayHiWrapper(*this, string, number);
 }
 
 void
 MyObject::sayBye()
 {
-  m_rustObj->sayBye(*this);
+  const std::lock_guard<std::mutex> guard(m_rustObjMutex);
+  m_rustObj->sayByeWrapper(*this);
 }
 
-std::unique_ptr<MyObject>
+std::unique_ptr<CppObj>
 newCppObject()
 {
-  return std::make_unique<MyObject>();
+  return std::make_unique<CppObj>();
 }
 
 } // namespace cxx_qt::my_object
